@@ -116,6 +116,7 @@ FindClusters <- function(object, ...) {
 #' @export
 #'
 #' @examples
+#' \dontrun{
 #' data("pbmc_small")
 #' # Find markers for cluster 2
 #' markers <- FindMarkers(object = pbmc_small, ident.1 = 2)
@@ -133,7 +134,8 @@ FindClusters <- function(object, ...) {
 #'   markers <- FindMarkers(object = pbmc_small, ident.1 = 'clustertree', ident.2 = 5)
 #'   head(x = markers)
 #' }
-#'
+#' }
+#' 
 #' @rdname FindMarkers
 #' @export FindMarkers
 #'
@@ -154,13 +156,13 @@ FindMarkers <- function(object, ...) {
 #' @param object An object
 #' @param ... Arguments passed to other methods
 #'
-#' @return This function can either return a \code{\link{Neighbor}} object
-#' with the KNN information or a list of \code{\link{Graph}} objects with
+#' @return This function can either return a \code{\link[SeuratObject]{Neighbor}} object
+#' with the KNN information or a list of \code{\link[SeuratObject]{Graph}} objects with
 #' the KNN and SNN depending on the settings of \code{return.neighbor} and
-#' \code{compute.SNN}. When running on a \code{\link{Seurat}} object, this
-#' returns the \code{\link{Seurat}} object with the Graphs or Neighbor objects
+#' \code{compute.SNN}. When running on a \code{\link[SeuratObject]{Seurat}} object, this
+#' returns the \code{\link[SeuratObject]{Seurat}} object with the Graphs or Neighbor objects
 #' stored in their respective slots. Names of the Graph or Neighbor object can
-#' be found with \code{\link{Graphs}} or \code{\link{Neighbors}}.
+#' be found with \code{\link[SeuratObject]{Graphs}} or \code{\link[SeuratObject]{Neighbors}}.
 #'
 #' @examples
 #' data("pbmc_small")
@@ -230,9 +232,11 @@ FindSpatiallyVariableFeatures <- function(object, ...) {
 #' Otherwise, log2 fold change is returned with column named "avg_log2_FC".
 #'
 #' @examples
+#' \dontrun{
 #' data("pbmc_small")
 #' FoldChange(pbmc_small, ident.1 = 1)
-#'
+#' }
+#' 
 #' @param object A Seurat object
 #' @param ... Arguments passed to other methods
 #' @rdname FoldChange
@@ -273,7 +277,7 @@ GetAssay <- function(object, ...) {
 #' @param reductions Name of reductions to be integrated. For a
 #' TransferAnchorSet, this should be the name of a reduction present in the
 #' anchorset object (for example, "pcaproject"). For an IntegrationAnchorSet,
-#' this should be a \code{\link{DimReduc}} object containing all cells present
+#' this should be a \code{\link[SeuratObject]{DimReduc}} object containing all cells present
 #' in the anchorset object.
 #' @param dims.to.integrate Number of dimensions to return integrated values for
 #' @param weight.reduction Dimension reduction to use when calculating anchor
@@ -283,7 +287,7 @@ GetAssay <- function(object, ...) {
 #'    all objects to be integrated}
 #'    \item{A vector of strings, specifying the name of a dimension reduction to
 #'    use for each object to be integrated}
-#'    \item{A vector of \code{\link{DimReduc}} objects, specifying the object to
+#'    \item{A vector of \code{\link[SeuratObject]{DimReduc}} objects, specifying the object to
 #'    use for each object in the integration}
 #'    \item{NULL, in which case the full corrected space is used for computing
 #'    anchor weights.}
@@ -302,6 +306,56 @@ IntegrateEmbeddings <- function(anchorset, ...) {
   UseMethod(generic = "IntegrateEmbeddings", object = anchorset)
 }
 
+#' Leverage Score Calculation
+#'
+#' This function computes the leverage scores for a given object
+#' It uses the concept of sketching and random projections. The function provides an approximation 
+#' to the leverage scores using a scalable method suitable for large matrices.
+#'
+#' @param object A matrix-like object
+#' @param ... Arguments passed to other methods
+#' 
+#' @references Clarkson, K. L. & Woodruff, D. P.
+#' Low-rank approximation and regression in input sparsity time.
+#' JACM 63, 1–45 (2017). \url{https://dl.acm.org/doi/10.1145/3019134};
+#'
+#' @export
+#'
+LeverageScore <- function(object, ...) {
+  UseMethod(generic = 'LeverageScore', object = object)
+}
+
+#' Normalize Raw Data
+#'
+#' @param data Matrix with the raw count data
+#' @param scale.factor Scale the data; default is \code{1e4}
+#' @param margin Margin to normalize over
+#' @param verbose Print progress
+#'
+#' @return A matrix with the normalized and log-transformed data
+#'
+#' @template param-dotsm
+#'
+#' @export
+#' @concept preprocessing
+#'
+#' @examples
+#' mat <- matrix(data = rbinom(n = 25, size = 5, prob = 0.2), nrow = 5)
+#' mat
+#' mat_norm <- LogNormalize(data = mat)
+#' mat_norm
+#'
+LogNormalize <- function(
+  data,
+  scale.factor = 1e4,
+  margin = 2L,
+  verbose = TRUE,
+  ...
+) {
+  UseMethod(generic = 'LogNormalize', object = data)
+}
+
+
 #' Metric for evaluating mapping success
 #'
 #' This metric was designed to help identify query cells that aren't well
@@ -340,6 +394,34 @@ NormalizeData <- function(object, ...) {
   UseMethod(generic = 'NormalizeData', object = object)
 }
 
+#' Project query data to the reference dimensional reduction
+#'
+#'
+#' @param query An object for query cells
+#' @param reference An object for reference cells
+#' @param query.assay Assay name for query object
+#' @param reference.assay Assay name for reference object
+#' @param reduction Name of dimensional reduction from reference object
+#' @param dims Dimensions used for reference dimensional reduction
+#' @param scale Determine if scale query data based on reference data variance
+#' @param verbose Print progress
+#' @param feature.mean Mean of features in reference
+#' @param feature.sd Standard variance of features in reference
+#'
+#' @return A matrix with projected cell embeddings
+#'
+#' @rdname ProjectCellEmbeddings
+#' @export ProjectCellEmbeddings
+#'
+#' @keywords internal
+#'
+ProjectCellEmbeddings <- function(
+  query,
+  ...
+) {
+  UseMethod(generic = 'ProjectCellEmbeddings', object = query)
+}
+
 #' Project query into UMAP coordinates of a reference
 #'
 #' This function will take a query dataset and project it into the coordinates
@@ -360,6 +442,23 @@ ProjectUMAP <- function(query, ...) {
   UseMethod(generic = "ProjectUMAP", object = query)
 }
 
+#' Pseudobulk Expression
+#'
+#' Normalize the count data present in a given assay.
+#'
+#' @param object An assay
+#' @param ... Arguments passed to other methods
+#'
+#' @return Returns object after normalization
+#'
+#' @rdname PseudobulkExpression
+#' @export PseudobulkExpression
+#' @concept utilities
+#'
+PseudobulkExpression <- function(object, ...) {
+  UseMethod(generic = "PseudobulkExpression", object = object)
+}
+
 #' Perform Canonical Correlation Analysis
 #'
 #' Runs a canonical correlation analysis using a diagonal implementation of CCA.
@@ -371,9 +470,10 @@ ProjectUMAP <- function(query, ...) {
 #'
 #' @return Returns a combined Seurat object with the CCA results stored.
 #'
-#' @seealso \code{\link{merge.Seurat}}
+#' @seealso \code{\link[SeuratObject]{merge.Seurat}}
 #'
 #' @examples
+#' \dontrun{
 #' data("pbmc_small")
 #' pbmc_small
 #' # As CCA requires two datasets, we will split our test object into two just for this example
@@ -384,7 +484,8 @@ ProjectUMAP <- function(query, ...) {
 #' pbmc_cca <- RunCCA(object1 = pbmc1, object2 = pbmc2)
 #' # Print results
 #' print(x = pbmc_cca[["cca"]])
-#'
+#' }
+#' 
 #' @rdname RunCCA
 #' @export RunCCA
 #'
@@ -392,6 +493,29 @@ RunCCA <- function(object1, object2, ...) {
   UseMethod(generic = 'RunCCA', object = object1)
 }
 
+
+#' Run Graph Laplacian Eigendecomposition
+#'
+#' Run a graph laplacian dimensionality reduction. It is used as a low
+#' dimensional representation for a cell-cell graph. The input graph
+#' should be symmetric
+#'
+#' @param object A Seurat object
+#' @param ... Arguments passed to
+#' \code{\link[RSpectra:eigs_sym]{RSpectra::eigs_sym}}
+#'
+#' @return Returns Seurat object with the Graph laplacian eigenvector
+#' calculation stored in the reductions slot
+#'
+#' @rdname RunGraphLaplacian
+#' @export RunGraphLaplacian
+#'
+
+RunGraphLaplacian <- function(object, ...) {
+  UseMethod(generic = 'RunGraphLaplacian', object = object)
+}
+
+
 #' Run Independent Component Analysis on gene expression
 #'
 #' Run fastica algorithm from the ica package for ICA dimensionality reduction.
@@ -476,7 +600,7 @@ RunSLSI <- function(object, ...) {
 #' @references Barshan E, Ghodsi A, Azimifar Z, Jahromi MZ.
 #' Supervised principal component analysis: Visualization, classification and
 #' regression on subspaces and submanifolds.
-#' Pattern Recognition. 2011 Jul 1;44(7):1357-71. \url{https://www.sciencedirect.com/science/article/pii/S0031320310005819?casa_token=AZMFg5OtPnAAAAAA:_Udu7GJ7G2ed1-XSmr-3IGSISUwcHfMpNtCj-qacXH5SBC4nwzVid36GXI3r8XG8dK5WOQui};
+#' Pattern Recognition. 2011 Jul 1;44(7):1357-71. \url{doi:10.1016/j.patcog.2010.12.015};
 #' @export
 #'
 #' @rdname RunSPCA
@@ -604,6 +728,17 @@ ScoreJackStraw <- function(object, ...) {
   UseMethod(generic = 'ScoreJackStraw', object = object)
 }
 
+#' Perform sctransform-based normalization
+#' @param object An object
+#' @param ... Arguments passed to other methods (not used)
+#'
+#' @rdname SCTransform
+#' @export SCTransform
+#'
+SCTransform <- function(object, ...) {
+  UseMethod(generic = 'SCTransform', object = object)
+}
+
 #' Get SCT results from an Assay
 #'
 #' Pull the \code{\link{SCTResults}} information from an \code{\link{SCTAssay}}
@@ -628,3 +763,46 @@ SCTResults <- function(object, ...) {
 "SCTResults<-" <- function(object, ..., value) {
   UseMethod(generic = 'SCTResults<-', object = object)
 }
+
+#' Variance Stabilizing Transformation
+#'
+#' Apply variance stabilizing transformation for selection of variable features
+#'
+#' @inheritParams stats::loess
+#' @param data A matrix-like object
+#' @param margin Unused
+#' @param nselect Number of of features to select
+#' @param clip Upper bound for values post-standardization; defaults to the
+#' square root of the number of cells
+#' @param verbose ...
+#'
+#' @template param-dotsm
+#'
+#' @return A data frame with the following columns:
+#' \itemize{
+#'  \item \dQuote{\code{mean}}: ...
+#'  \item \dQuote{\code{variance}}: ...
+#'  \item \dQuote{\code{variance.expected}}: ...
+#'  \item \dQuote{\code{variance.standardized}}: ...
+#'  \item \dQuote{\code{variable}}: \code{TRUE} if the feature selected as
+#'   variable, otherwise \code{FALSE}
+#'  \item \dQuote{\code{rank}}: If the feature is selected as variable, then how
+#'   it compares to other variable features with lower ranks as more variable;
+#'   otherwise, \code{NA}
+#' }
+#'
+#' @rdname VST
+#' @export VST
+#'
+#' @keywords internal
+#'
+VST <- function(
+  data,
+  margin = 1L,
+  nselect = 2000L,
+  span = 0.3,
+  clip = NULL,
+  ...
+) {
+  UseMethod(generic = 'VST', object = data)
+}
