@@ -249,7 +249,17 @@ test_that("t test works", {
   expect_equal(rownames(x = results)[1], "TYMP")
 })
 
-results <- suppressWarnings(FindMarkers(object = pbmc_small, ident.1 = 0, ident.2 = 1, test.use = "negbinom", verbose = FALSE, base = exp(1), pseudocount.use = 1))
+results <- suppressWarnings(
+  FindMarkers(
+    object = pbmc_small,
+    ident.1 = 0, ident.2 = 1,
+    test.use = "negbinom",
+    verbose = FALSE,
+    base = exp(1),
+    fc.slot = "counts",
+    pseudocount.use = 1
+  )
+)
 test_that("negbinom test works", {
   expect_equal(nrow(x = results), 204)
   expect_equal(results["CST3", "p_val"], 1.354443e-17, tolerance = 1e-22)
@@ -260,7 +270,18 @@ test_that("negbinom test works", {
   expect_equal(rownames(x = results)[1], "LYZ")
 })
 
-results <- suppressWarnings(FindMarkers(object = pbmc_small, ident.1 = 0, ident.2 = 1, test.use = "poisson", verbose = FALSE, base = exp(1), pseudocount.use = 1))
+results <- suppressWarnings(
+  FindMarkers(
+    object = pbmc_small,
+    ident.1 = 0,
+    ident.2 = 1,
+    test.use = "poisson",
+    verbose = FALSE,
+    base = exp(1),
+    fc.slot = "counts",
+    pseudocount.use = 1
+  )
+)
 test_that("poisson test works", {
   expect_equal(nrow(x = results), 204)
   expect_equal(results["CST3", "p_val"], 3.792196e-78, tolerance = 1e-83)
@@ -345,12 +366,17 @@ test_that("BPCells FindMarkers gives same results", {
 
 # Tests for FindAllMarkers
 # -------------------------------------------------------------------------------
-results <- suppressMessages(suppressWarnings(FindAllMarkers(object = pbmc_small,pseudocount.use=1)))
-results.clr <- suppressMessages(suppressWarnings(FindAllMarkers(object = clr.obj, pseudocount.use=1)))
-results.sct <- suppressMessages(suppressWarnings(FindAllMarkers(object = sct.obj, pseudocount.use=1, vst.flavor = "v1")))
-results.pseudo <- suppressMessages(suppressWarnings(FindAllMarkers(object = pbmc_small, pseudocount.use = 0.1)))
 
 test_that("FindAllMarkers works as expected", {
+  pbmc_copy <- pbmc_small
+  Idents(pbmc_copy) <- "orig.ident"
+
+  results <- suppressMessages(suppressWarnings(FindAllMarkers(object = pbmc_small, pseudocount.use = 1)))
+  results.clr <- suppressMessages(suppressWarnings(FindAllMarkers(object = clr.obj, pseudocount.use = 1)))
+  results.sct <- suppressMessages(suppressWarnings(FindAllMarkers(object = sct.obj, pseudocount.use = 1, vst.flavor = "v1")))
+  results.pseudo <- suppressMessages(suppressWarnings(FindAllMarkers(object = pbmc_small, pseudocount.use = 0.1)))
+  results.gb <- suppressMessages(suppressWarnings(FindAllMarkers(object = pbmc_copy, pseudocount.use = 1, group.by = "RNA_snn_res.1")))
+
   expect_equal(colnames(x = results), c("p_val", "avg_log2FC", "pct.1", "pct.2", "p_val_adj", "cluster", "gene"))
   expect_equal(results[1, "p_val"], 9.572778e-13, tolerance = 1e-18)
   expect_equal(results[1, "avg_log2FC"], -6.030507, tolerance = 1e-6)
@@ -386,6 +412,10 @@ test_that("FindAllMarkers works as expected", {
   expect_equal(results.pseudo[1, "p_val_adj"], 2.201739e-10, tolerance = 1e-15)
   expect_equal(nrow(x = results.pseudo), 222)
   expect_equal(rownames(results.pseudo)[1], "HLA-DPB1")
+
+  # Setting `group.by` the group by parameter is equivalent
+  # to setting the object's `Idents` before running `FindAllMarkers`.
+  expect_equal(results.gb, results)
 })
 
 
@@ -409,7 +439,7 @@ query.list <- lapply(X = query.list, FUN = ScaleData, verbose = FALSE)
 query.list <- suppressWarnings(lapply(X = query.list, FUN = RunPCA, verbose = FALSE, npcs = 20))
 
 anchors <- suppressMessages(suppressWarnings(FindIntegrationAnchors(object.list = c(ref, query.list), k.filter = NA, verbose = FALSE)))
-object <- suppressMessages(IntegrateData(anchorset = anchors,  k.weight = 25, verbose = FALSE))
+object <- suppressWarnings(suppressMessages(IntegrateData(anchorset = anchors,  k.weight = 25, verbose = FALSE)))
 object <- suppressMessages(ScaleData(object, verbose = FALSE))
 object <- suppressMessages(RunPCA(object, verbose = FALSE))
 object <- suppressMessages(FindNeighbors(object = object, verbose = FALSE))
